﻿using SampleApp.Business.Exceptions;
using SampleApp.Common.Extensions;
using SampleApp.Data.Common;
using SampleApp.Data.Context;
using SampleApp.Data.Exceptions;

namespace SampleApp.Business.Crud.Common
{
    internal abstract class CommandFacadeBase<TEntity> : ICommandFacade
        where TEntity : class
    {
        private readonly ISampleAppContextFactory _contextFactory;

        protected CommandFacadeBase(
            ISampleAppContextFactory contextFactory,
            ICreateEntityOperation<TEntity> createOperation,
            IUpdateEntityOperation<TEntity> updateOperation,
            IDeleteEntityOperation<TEntity> deleteOperation)
        {
            _contextFactory = contextFactory;
            CreateOperation = createOperation;
            UpdateOperation = updateOperation;
            DeleteOperation = deleteOperation;
        }

        protected ICreateEntityOperation<TEntity> CreateOperation { get; set; }
        protected IUpdateEntityOperation<TEntity> UpdateOperation { get; set; }
        protected IDeleteEntityOperation<TEntity> DeleteOperation { get; set; }

        public bool Create<TDto>(TDto dto)
        {
            if (CreateOperation == null)
                throw new SampleAppBusinessException("Vložení nové entity tohoto typu není povoleno!");

            ICrudOperation operation = CreateOperation;

            var failMessage = "Uložení nového objektu {0} do databáze se nezdařilo.".LocalFormat(dto);
            return Execute(dto, operation, failMessage);
        }

        public bool Update<TDto>(TDto dto)
        {
            if (UpdateOperation == null)
                throw new SampleAppBusinessException("Změna entity tohoto typu není povolena!");

            var operation = UpdateOperation;

            var failMessage = "Aktualizace objektu {0} v databázi se nezdařilo.".LocalFormat(dto);
            return Execute(dto, operation, failMessage);
        }

        public bool Delete(int id)
        {
            if (DeleteOperation == null)
                throw new SampleAppBusinessException("Smazání entity tohoto typu není povoleno!");

            var validationResult = new DataValidationResult();
            var context = _contextFactory.ResolveContext();
            var ok = DeleteOperation.Execute(id, context, validationResult);

            if (validationResult.IsError)
                throw new DataValidationException(validationResult);

            if (!ok)
            {
                var failMessage = "Smazání objektu číslo {0} z databáze se nezdařilo.".LocalFormat(id);
                throw new SampleAppBusinessException(failMessage);
            }

            return true;
        }

        private bool Execute<TDto>(TDto dto, ICrudOperation operation, string failMessage)
        {
            var validationResult = new DataValidationResult();
            var context = _contextFactory.ResolveContext();
            var ok = operation.Execute(dto, context, validationResult);

            if (validationResult.IsError)
                throw new DataValidationException(validationResult);

            if (!ok)
                throw new SampleAppBusinessException(failMessage);

            return true;
        }
    }
}