﻿using System;
using System.Collections.Generic;
using System.Data.Entity;

using JetBrains.Annotations;

using SampleApp.Business.Crud.Validators;
using SampleApp.Common.Extensions;
using SampleApp.Common.Interfaces;
using SampleApp.Data.Common;
using SampleApp.Data.SampleAppEntities;

namespace SampleApp.Business.Crud.Common
{
    // ReSharper disable once UnusedTypeParameter
    internal interface ICreateEntityOperation<TEntity> : IService, ICrudOperation
        where TEntity : class
    {
    }

    internal class CreateEntityOperation<TEntity> : CommandOperationBase, ICreateEntityOperation<TEntity>
        where TEntity : class
    {
        private readonly IEnumerable<IEntityValidator<TEntity>> _validators;

        public CreateEntityOperation(IEnumerable<IEntityValidator<TEntity>> validators)
        {
            _validators = validators;
        }

        public bool Execute<TDto>(TDto dto, [NotNull] SampleAppContext context, [NotNull] DataValidationResult validationResult)
        {
            if (context == null)
                throw new ArgumentNullException("context");
            if (validationResult == null)
                throw new ArgumentNullException("validationResult");

            TEntity entity = null;
            try
            {
                ValidateDto(dto, context, validationResult);
                if (validationResult.IsError)
                    return false;

                entity = MappingExtensions.MapIfTypesDiffers<TDto, TEntity>(dto);

                ValidateEntity(entity, context, validationResult);
                if (validationResult.IsError)
                    return false;

                var entry = context.Entry(entity);
                entry.State = EntityState.Added;
                var cnt = context.SaveChanges();
                entry.State = EntityState.Detached;
                return cnt > 0;
            }
            catch (Exception ex)
            {
                var handled = HandleDbException(ex, entity as object ?? dto);
                if (!handled)
                    throw;
                return false;
            }
        }

        protected virtual void ValidateEntity(TEntity entity, SampleAppContext context, DataValidationResult validationResult)
        {
            foreach (var validator in _validators)
            {
                validator.ValidateNew(entity, context, validationResult);
            }
        }
    }
}