﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;

using JetBrains.Annotations;

using SampleApp.Business.Crud.Validators;
using SampleApp.Business.Exceptions;
using SampleApp.Common.Extensions;
using SampleApp.Common.Interfaces;
using SampleApp.Data.Common;
using SampleApp.Data.SampleAppEntities;

namespace SampleApp.Business.Crud.Common
{
    // ReSharper disable once UnusedTypeParameter
    internal interface IDeleteEntityOperation<TEntity> : IService
    {
        bool Execute(int id, SampleAppContext context, [NotNull] DataValidationResult validationResult);
    }

    internal class DeleteEntityOperation<TEntity> : IDeleteEntityOperation<TEntity>
        where TEntity : class, IPrimaryKey, new()
    {
        private readonly IEnumerable<IEntityValidator<TEntity>> _validators;

        public DeleteEntityOperation(IEnumerable<IEntityValidator<TEntity>> validators)
        {
            _validators = validators;
        }

        public bool Execute(int id, SampleAppContext context, DataValidationResult validationResult)
        {
            if (validationResult == null)
                throw new ArgumentNullException("validationResult");

            try
            {
                ValidateDelete(id, context, validationResult);
                if (validationResult.IsError)
                    return false;

                var entity = new TEntity { Id = id };
                context.TryDetachEntity(entity);
                var entry = context.Entry(entity);
                entry.State = EntityState.Deleted;
                var count = context.SaveChanges();
                entry.State = EntityState.Detached;
                return count > 0;
            }
            catch (Exception ex)
            {
                if (ex is DbUpdateConcurrencyException)
                    return false;

                var message = "Failed to process entity {0}.".InvariantFormat(id);
                throw new SampleAppBusinessException(message, ex);
            }
        }

        protected virtual void ValidateDelete(int id, SampleAppContext context, DataValidationResult validationResult)
        {
            foreach (var validator in _validators)
            {
                validator.ValidateDelete(id, context, validationResult);
            }
        }
    }
}
