﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;
using System.Linq.Expressions;

using AutoMapper;

using JetBrains.Annotations;

using SampleApp.Business.Crud.Validators;
using SampleApp.Common.Interfaces;
using SampleApp.Data.Common;
using SampleApp.Data.SampleAppEntities;

namespace SampleApp.Business.Crud.Common
{
    // ReSharper disable once UnusedTypeParameter
    internal interface IUpdateEntityOperation<TEntity> : IService, ICrudOperation
    {
    }

    internal abstract class UpdateEntityOperationBase<TEntity> : CommandOperationBase, IUpdateEntityOperation<TEntity>
        where TEntity : class, new()
    {
        private readonly IEnumerable<IEntityValidator<TEntity>> _validators;

        protected UpdateEntityOperationBase(IEnumerable<IEntityValidator<TEntity>> validators)
        {
            _validators = validators;
        }

        public bool Execute<TDto>(TDto dto, SampleAppContext context, [NotNull] DataValidationResult validationResult)
        {
            if (validationResult == null)
                throw new ArgumentNullException("validationResult");

            TEntity entity = null;
            try
            {
                ValidateDto(dto, context, validationResult);
                if (validationResult.IsError)
                    return false;

                entity = Mapper.Map<TEntity>(dto);

                context.TryDetachEntity(entity);
                var entry = context.Entry(entity);
                entry.State = EntityState.Unchanged;

                ValidateEntity(entity, context, validationResult);
                if (validationResult.IsError)
                {
                    entry.State = EntityState.Detached;
                    return false;
                }

                SetUpdateableProperties(entry);

                var changesCount = context.SaveChanges();
                entry.State = EntityState.Detached;
                return changesCount > 0;
            }
            catch (Exception ex)
            {
                var handled = HandleDbException(ex, entity as object ?? dto);
                if (!handled)
                    throw;
                return false;
            }
        }

        protected virtual void ValidateEntity(TEntity entity, SampleAppContext context, DataValidationResult validationResult)
        {
            foreach (var validator in _validators)
            {
                validator.ValidateUpdate(entity, context, validationResult);
            }
        }

        protected abstract void SetUpdateableProperties(DbEntityEntry<TEntity> entry);

        protected void SetModified<TProperty>(DbEntityEntry<TEntity> entityEntry, params Expression<Func<TEntity, TProperty>>[] propertyGetters)
        {
            foreach (var propertyGetter in propertyGetters)
            {
                entityEntry.Property(propertyGetter).IsModified = true;
            }
        }
    }
}
