﻿using System;
using System.Globalization;
using System.Linq;

using JetBrains.Annotations;

using SampleApp.Common.Exceptions;

namespace SampleApp.Common.Extensions
{
    public static class StringExtensions
    {
        /// <summary>
        ///     Extension variant for <see cref="string.Format(string,object)" />, with handled FormatException.
        /// </summary>
        /// <param name="pattern">placeholder text to format</param>
        /// <param name="culture">Culture for ToString</param>
        /// <param name="args">arguments to be passed into plceholders</param>
        /// <returns>Text formatted by standardn <c>String.Format()</c> method.</returns>
        ////[StringFormatMethod("pattern")]
        public static string SFormat(this string pattern, CultureInfo culture, params object[] args)
        {
            try
            {
                return string.Format(CultureInfo.CurrentCulture, pattern, args);
            }
            catch (FormatException ex)
            {
                var msg = "Format string is invalid: pattern=[{0}], args=[{1}]";
                if (args == null)
                {
                    args = new object[0];
                }

                msg = string.Format(culture, msg, pattern, string.Join("; ", args));
                throw new SampleAppCommonException(msg, ex);
            }
        }

        /// <summary>
        ///     Extension variant for <see cref="string.Format(string,object)" />, with handled FormatException and <see cref="CultureInfo.InvariantCulture" />.
        /// </summary>
        /// <param name="pattern">placeholder text to format</param>
        /// <param name="args">arguments to be passed into plceholders</param>
        /// <returns>Text formatted by standardn <c>String.Format()</c> method.</returns>
        [StringFormatMethod("pattern")]
        public static string InvariantFormat(this string pattern, params object[] args)
        {
            return pattern.SFormat(CultureInfo.InvariantCulture, args);
        }

        /// <summary>
        ///     Extension variant for <see cref="string.Format(string,object)" />, with handled FormatException and <see cref="CultureInfo.CurrentCulture" />.
        /// </summary>
        /// <param name="pattern">placeholder text to format</param>
        /// <param name="args">arguments to be passed into plceholders</param>
        /// <returns>Text formatted by standardn <c>String.Format()</c> method.</returns>
        [StringFormatMethod("pattern")]
        public static string LocalFormat(this string pattern, params object[] args)
        {
            return pattern.SFormat(CultureInfo.CurrentCulture, args);
        }

        public static bool IsNullOrEmpty(this string value)
        {
            return string.IsNullOrEmpty(value);
        }

        public static bool IsNullOrWhiteSpace(this string value)
        {
            return string.IsNullOrWhiteSpace(value);
        }

        public static string StripSpecialChars([NotNull] this string text, char replacement)
        {
            if (text == null)
                throw new ArgumentNullException("text");
            var result = text.Select(x => char.IsLetterOrDigit(x) ? x : replacement);
            return new string(result.ToArray());
        }
    }
}
