﻿using System;
using System.ComponentModel.DataAnnotations.Schema;
using System.Data.Entity;
using System.Data.Entity.Core.Metadata.Edm;
using System.Data.Entity.Infrastructure;
using System.Data.Entity.Validation;
using System.Diagnostics.CodeAnalysis;
using System.Linq;

using SampleApp.Common.Extensions;
using SampleApp.Common.Infrastructure;
using SampleApp.Data.Common;
using SampleApp.Data.Exceptions;

namespace SampleApp.Data.SampleAppEntities
{
    public partial class SampleAppContext
    {
        public override int SaveChanges()
        {
            if (Configuration.AutoDetectChangesEnabled)
            {
                UpdatedTrackableFields();

                if (Configuration.ValidateOnSaveEnabled)
                {
                    throw new SampleAppDataException("Configuration.ValidateOnSaveEnabled must be set to false in order to make it working...");
                }

                var errorResults = GetValidationErrors().ToArray();
                if (errorResults.Any())
                {
                    ProcessValidations(errorResults);
                }
            }

            return base.SaveChanges();
        }

        [SuppressMessage("StyleCop.CSharp.OrderingRules", "SA1202:ElementsMustBeOrderedByAccess", Justification = "Reviewed. Suppression is OK here.")]
        public void TryDetachEntity<TEntity>(TEntity entity)
        {
            var objectCtx = ((IObjectContextAdapter)this).ObjectContext;
            var container = objectCtx.MetadataWorkspace.GetEntityContainer(objectCtx.DefaultContainerName, DataSpace.CSpace);
            var entitySetBase = container.BaseEntitySets.Single(item => item.ElementType.Name.Equals(typeof(TEntity).Name));
            var entityKey = objectCtx.CreateEntityKey(entitySetBase.Name, entity);
            object oldEntity;
            if (objectCtx.TryGetObjectByKey(entityKey, out oldEntity))
                objectCtx.Detach(oldEntity);
        }

        private void ProcessValidations(DbEntityValidationResult[] errorResults)
        {
            var dataValidations = new DataValidationResult();
            foreach (var result in errorResults)
            {
                foreach (var error in result.ValidationErrors)
                {
                    if (result.Entry.Entity is ITrackableEntity && error.PropertyName == "CreatedBy")
                    {
                        continue;
                    }

                    var message = "{0}.{1}: {2}".InvariantFormat(result.Entry.Entity, error.PropertyName, error.ErrorMessage);
                    dataValidations.AddError(message);
                }
            }

            if (dataValidations.IsError)
            {
                throw new DataValidationException(dataValidations);
            }
        }

        private void UpdatedTrackableFields()
        {
            var trackables = ChangeTracker.Entries<ITrackableEntity>()
                .Where(e => e.State.In(EntityState.Added, EntityState.Modified, EntityState.Deleted))
                .ToArray();
            
            if (trackables.Any())
            {
                var user = _authenticationService.CurrentUser;
                if (user == null || user.Identity == null || user.Identity.Name == null)
                    throw new SampleAppDataException("Neznámý uživatel, nelze zapsat změnu do databáze!");
                string username = "anonymous";
                if (user.Identity != null && user.Identity.IsAuthenticated)
                    username = user.Identity.Name;
                var now = _dateTimeFacade.GetCurrent();

                foreach (var item in trackables.Where(t => t.State == EntityState.Added))
                {
                    var entity = item.Entity;

                    entity.CreatedAt = now;
                    entity.CreatedBy = username;
                    entity.ModifiedAt = null;
                    entity.ModifiedBy = null;
                }

                foreach (var item in trackables.Where(t => t.State.In(EntityState.Modified)))
                {
                    var entity = item.Entity;

                    entity.ModifiedAt = now;
                    entity.ModifiedBy = username;

                    item.Property(i => i.CreatedAt).IsModified = false;
                    item.Property(i => i.CreatedBy).IsModified = false;
                }
            }
        }
    }
}
