﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Web;
using System.Web.Mvc;

using AutoMapper;

using SampleApp.Business.Crud.Common;
using SampleApp.Business.Exceptions;
using SampleApp.Business.Queries;
using SampleApp.Common.Extensions;
using SampleApp.Data.Common;
using SampleApp.Data.Exceptions;
using SampleApp.WebUI.Infrastructure.Caching;
using SampleApp.WebUI.Models;

namespace SampleApp.WebUI.Controllers
{
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1005:AvoidExcessiveParametersOnGenericTypes")]
    [NoCache]
    public abstract class CommandControllerBase<TBusinessDto, TViewModel>
        : MvcControllerBase
        where TBusinessDto : class, IPrimaryKey
        where TViewModel : class, IPrimaryKey, new()
    {
        private readonly IQuery<TBusinessDto> _detailsQuery;

        protected CommandControllerBase(
            IQuery<TBusinessDto> detailsQuery,
            ICommandFacade commandFacade)
        {
            _detailsQuery = detailsQuery;
            CommandFacade = commandFacade;
        }

        protected ICommandFacade CommandFacade { get; private set; }

        [HttpGet]
        public virtual ActionResult Create()
        {
            var model = new TViewModel();
            return View(model);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public virtual ActionResult Create(TViewModel model)
        {
            if (ModelState.IsValid)
            {
                var dto = MappingExtensions.MapIfTypesDiffers<TViewModel, TBusinessDto>(model);
                var ok = false;
                try
                {
                    ok = CommandFacade.Create(dto);
                }
                catch (DataValidationException ex)
                {
                    AddMessage(MessageTarget.PageData, MessageType.Error, ex.Message);
                }

                if (ok)
                {
                    AddMessage(MessageTarget.PageData, MessageType.Ok, "Záznam byl uložen.");
                    return RedirectToAction("Index", "PersonList");
                }
                else
                {
                    AddMessage(MessageTarget.PageData, MessageType.Error, "Uložení se nepodařilo.");
                }
            }

            return View(model);
        }

        [HttpGet]
        public virtual ActionResult Edit(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.InternalServerError, "Nebyl předán povinný parametr [id] typu integer.");
            var dto = _detailsQuery.GetOne(id.Value);
            if (dto == null)
                return HttpNotFound("Záznam nelze smazat, protože nebyl nalezen v databázi.");
            return View(dto);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public virtual ActionResult Edit(TViewModel model)
        {
            if (ModelState.IsValid)
            {
                var dto = MappingExtensions.MapIfTypesDiffers<TViewModel, TBusinessDto>(model);
                var ok = false;
                try
                {
                    ok = CommandFacade.Update(dto);
                    
                    // reload the entity from DB after successfull save operation to see what was really saved:
                    dto = _detailsQuery.GetOne(dto.Id);
                    model = MappingExtensions.MapIfTypesDiffers<TBusinessDto, TViewModel>(dto);
                }
                catch (DataValidationException ex)
                {
                    AddMessage(MessageTarget.PageData, MessageType.Error, ex.Message);
                }

                if (ok)
                    AddMessage(MessageTarget.PageData, MessageType.Ok, "Záznam byl uložen.");
                else
                    AddMessage(MessageTarget.PageData, MessageType.Error, "Uložení se nepodařilo.");
            }

            return View(model);
        }

        [HttpGet]
        public virtual ActionResult DeletePreview(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.InternalServerError, "Nebyl předán povinný parametr [id] typu integer.");
            var dto = _detailsQuery.GetOne(id.Value);
            if (dto == null)
                return HttpNotFound("Záznam nelze smazat, protože nebyl nalezen v databázi.");
            return View(dto);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public virtual ActionResult DeleteConfirmed(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.InternalServerError, "Nebyl předán povinný parametr [id] typu integer.");
            if (ModelState.IsValid)
            {
                var ok = false;
                try
                {
                    ok = CommandFacade.Delete(id.Value);
                }
                catch (DataValidationException ex)
                {
                    AddMessage(MessageTarget.PageData, MessageType.Error, ex.Message);
                }

                if (ok)
                    AddMessage(MessageTarget.PageData, MessageType.Ok, "Záznam byl smazán.");
                else
                    AddMessage(MessageTarget.PageData, MessageType.Error, "Smazání se nepodařilo.");
            }

            return RedirectToAction("Index", "PersonList");
        }
    }
}