﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;

using JetBrains.Annotations;

using SampleApp.Business.Exceptions;
using SampleApp.Common.Extensions;
using SampleApp.Data.Common;
using SampleApp.Data.Exceptions;
using SampleApp.WebUI.Infrastructure.Caching;
using SampleApp.WebUI.Models;

namespace SampleApp.WebUI.Controllers
{
    [NoCache]
    // [HandleConcurrencyException]
    public abstract class MvcControllerBase : Controller
    {
        public const string MessagesKey = "Messages";

        public bool ValidationExceptionSafeAction(MessageTarget target, [NotNull] Action action)
        {
            if (action == null) throw new ArgumentNullException("action");

            try
            {
                action();
                return true;
            }
            catch (DataValidationException ex)
            {
                foreach (var msg in ex.ValidationMessages)
                {
                    AddMessage(target, msg);
                }
            }

            return false;
        }

        // Message is displayed to user in this respone only.
        protected void AddMessage(MessageTarget target, MessageType messageType, string message)
        {
            var msg = new TextMessage(messageType, message);
            AddMessage(target, msg);
        }

        protected void AddMessage(MessageTarget target, [NotNull] IMessage message)
        {
            if (message == null) throw new ArgumentNullException("message");

            var messageList = new List<IMessage>();
            switch (target)
            {
                case MessageTarget.PageData:
                    if (!ViewData.ContainsKey(MessagesKey)) ViewData.Add(MessagesKey, messageList);
                    else messageList = (List<IMessage>)ViewData[MessagesKey];

                    messageList.Add(message);
                    break;
                case MessageTarget.ModelState:
                    if (message.Type == MessageType.Error)
                    {
                        ModelState.AddModelError(string.Empty, message.ToString());
                    }
                    else
                    {
                        // TODO: implement displaying Ok, Info and warning messages ...
                    }

                    break;
                default:
                    throw new NotSupportedException(target.ToString());
            }
        }

        [StringFormatMethod("message")]
        protected void AddMessage(MessageTarget target, MessageType messageType, string message, params object[] args)
        {
            if (args != null && args.Any())
            {
                message = message.LocalFormat(args);
            }

            AddMessage(target, messageType, message);
        }

        protected override RedirectResult Redirect(string url)
        {
            MessagesToTempData();
            return base.Redirect(url);
        }

        protected override RedirectResult RedirectPermanent(string url)
        {
            MessagesToTempData();
            return base.RedirectPermanent(url);
        }

        protected override RedirectToRouteResult RedirectToAction(string actionName, string controllerName, RouteValueDictionary routeValues)
        {
            MessagesToTempData();
            return base.RedirectToAction(actionName, controllerName, routeValues);
        }

        protected override RedirectToRouteResult RedirectToActionPermanent(string actionName, string controllerName, RouteValueDictionary routeValues)
        {
            MessagesToTempData();
            return base.RedirectToActionPermanent(actionName, controllerName, routeValues);
        }

        protected override RedirectToRouteResult RedirectToRoute(string routeName, RouteValueDictionary routeValues)
        {
            MessagesToTempData();
            return base.RedirectToRoute(routeName, routeValues);
        }

        protected override RedirectToRouteResult RedirectToRoutePermanent(string routeName, RouteValueDictionary routeValues)
        {
            MessagesToTempData();
            return base.RedirectToRoutePermanent(routeName, routeValues);
        }

        private void MessagesToTempData()
        {
            if (ViewData.ContainsKey(MessagesKey))
            {
                var messageList = (List<IMessage>)ViewData[MessagesKey];
                TempData.Add(MessagesKey, messageList);
                ViewData.Remove(MessagesKey);
            }
        }
    }
}